//
//  TextLCDController.m
//  TextLCD
//
//  Created by Phidgets on 2015-07-29.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "TextLCDController.h"

@implementation TextLCDController

- (void)windowWillClose:(NSNotification*)aNotification{
    if(ch){
        //Safely close the example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetLCD_delete(&ch);
        ch = NULL;
    }
}

-(void)awakeFromNib{
    PhidgetReturnCode result;
    
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetLCD_create(&ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
    [inputText0 setDelegate:self];
    [inputText1 setDelegate:self];
}

#pragma mark Event callbacks
static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICEREMOTE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}

#pragma mark Attach, detach, and error events
int width;
Phidget_DeviceID deviceID;
- (void)onAttachHandler{
    double contrast, backlight;
    PhidgetLCD_ScreenSize screenSize;
    //Get information about device
    Phidget_getDeviceID((PhidgetHandle)ch, &deviceID);
    PhidgetLCD_getContrast(ch, &contrast);
    PhidgetLCD_getBacklight(ch, &backlight);
    PhidgetLCD_getWidth(ch, &width);
    PhidgetLCD_getScreenSize(ch, &screenSize);
    
    //Adjust GUI based on information read from channel
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [settingsBox setHidden:NO];
    [contrastSlider setDoubleValue:contrast];
    [brightnessSlider setDoubleValue:backlight];
    
    switch(deviceID){
        case PHIDID_1202_1203:
            [screenSizeCombo setHidden:YES];
            [initButton setHidden:YES];
            break;
        default:
            [screenSizeCombo setHidden:NO];
            [screenSizeCombo insertItemWithTitle:@"No Screen" atIndex:0];
            [screenSizeCombo insertItemWithTitle:@"1x8" atIndex:1];
            [screenSizeCombo insertItemWithTitle:@"2x8" atIndex:2];
            [screenSizeCombo insertItemWithTitle:@"1x16" atIndex:3];
            [screenSizeCombo insertItemWithTitle:@"2x16" atIndex:4];
            [screenSizeCombo insertItemWithTitle:@"4x16" atIndex:5];
            [screenSizeCombo insertItemWithTitle:@"2x20" atIndex:6];
            [screenSizeCombo insertItemWithTitle:@"4x20" atIndex:7];
            [screenSizeCombo insertItemWithTitle:@"2x24" atIndex:8];
            [screenSizeCombo insertItemWithTitle:@"1x40" atIndex:9];
            [screenSizeCombo insertItemWithTitle:@"2x40" atIndex:10];
            [screenSizeCombo insertItemWithTitle:@"4x40" atIndex:11];
            [screenSizeCombo selectItemAtIndex:6];
            PhidgetLCD_setScreenSize(ch, SCREEN_SIZE_2x20);
            break;
    }
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow maxSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}


- (void)onDetachHandler{
    //Reset
    [settingsBox setHidden:YES];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [inputText0 setStringValue:@""];
    [inputText1 setStringValue:@""];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

static int errorCounter = 0;
-(void) outputError:(const char *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

-(void)errorHandler:(NSArray *)errorEventData{
    const char* errorString = [[errorEventData objectAtIndex:1] UTF8String];
    [self outputError:errorString];
}

-(void)addError:(PhidgetReturnCode)result{
    const char *errorString;
    Phidget_getErrorDescription(result, &errorString);
    [self outputError:errorString];
}

- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}


#pragma GUI controls
- (void) controlTextDidChange: (NSNotification *)aNotification{
    PhidgetReturnCode result;
    NSTextField *textfield = [aNotification object];
    NSString *saveString;
    int index = (int)textfield.tag;
    if(index == 0)
        saveString = [inputText1 stringValue];
    else
        saveString = [inputText0 stringValue];
    NSString *text = textfield.stringValue;
    PhidgetLCD_clear(ch);
    result = PhidgetLCD_writeText(ch,FONT_5x8,0,index == 1 ? 0 : 1, [saveString UTF8String]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    result = PhidgetLCD_writeText(ch,FONT_5x8,0,index, [text UTF8String]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    result = PhidgetLCD_flush(ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
}

- (IBAction)setContrast:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetLCD_setContrast(ch, [contrastSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
}

- (IBAction)setBrightness:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetLCD_setBacklight(ch, [brightnessSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
}

- (IBAction)setScreenSize:(id)sender{
    PhidgetReturnCode result;
    if([[screenSizeCombo titleOfSelectedItem]  isEqual: @"No Screen"])
        result = PhidgetLCD_setScreenSize(ch, SCREEN_SIZE_NONE);
    else if([[screenSizeCombo titleOfSelectedItem] isEqual:@"1x8"])
        result = PhidgetLCD_setScreenSize(ch, SCREEN_SIZE_1x8);
    else if([[screenSizeCombo titleOfSelectedItem] isEqual:@"2x8"])
        result = PhidgetLCD_setScreenSize(ch, SCREEN_SIZE_2x8);
    else if([[screenSizeCombo titleOfSelectedItem] isEqual:@"1x16"])
        result = PhidgetLCD_setScreenSize(ch, SCREEN_SIZE_1x16);
    else if([[screenSizeCombo titleOfSelectedItem] isEqual:@"2x16"])
        result = PhidgetLCD_setScreenSize(ch, SCREEN_SIZE_2x16);
    else if([[screenSizeCombo titleOfSelectedItem] isEqual:@"4x16"])
        result = PhidgetLCD_setScreenSize(ch, SCREEN_SIZE_4x16);
    else if([[screenSizeCombo titleOfSelectedItem] isEqual:@"2x20"])
        result = PhidgetLCD_setScreenSize(ch, SCREEN_SIZE_2x20);
    else if([[screenSizeCombo titleOfSelectedItem] isEqual:@"4x20"])
        result = PhidgetLCD_setScreenSize(ch, SCREEN_SIZE_4x20);
    else if([[screenSizeCombo titleOfSelectedItem] isEqual:@"2x24"])
        result = PhidgetLCD_setScreenSize(ch, SCREEN_SIZE_2x24);
    else if([[screenSizeCombo titleOfSelectedItem] isEqual:@"1x40"])
        result = PhidgetLCD_setScreenSize(ch, SCREEN_SIZE_1x40);
    else if([[screenSizeCombo titleOfSelectedItem] isEqual:@"2x40"])
        result = PhidgetLCD_setScreenSize(ch, SCREEN_SIZE_2x40);
    else if([[screenSizeCombo titleOfSelectedItem] isEqual:@"4x40"])
        result = PhidgetLCD_setScreenSize(ch, SCREEN_SIZE_4x40);
	else
		result = EPHIDGET_UNEXPECTED;
    
    if(result != EPHIDGET_OK){
        [self addError:result];
    }

}

- (IBAction)initialize:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetLCD_initialize(ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
}

- (IBAction)clear:(id)sender{
    PhidgetReturnCode result;
    [inputText0 setStringValue:@""];
    [inputText1 setStringValue:@""];
    
    result = PhidgetLCD_clear(ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    result = PhidgetLCD_flush(ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
}

- (IBAction)setCursor:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetLCD_setCursorOn(ch, [sender intValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
}

- (IBAction)setCursorBlink:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetLCD_setCursorBlink(ch, [sender intValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
}

-(uint8_t*)getBitmapBytes:(NSMutableArray*)arr{
    static uint8_t bytes[40];
    int j = 0;
    for(NSString *string in arr){
        for(int i = 0; i < string.length/2; i++){
            bytes[j++] = (uint8_t)[string characterAtIndex:i*2] == ' ' ? 0x00 : 0x01;
        }
    }
    return bytes;
}

- (IBAction)setCustomCharacters:(id)sender{
    PhidgetReturnCode result;
    NSButton *button = (NSButton *)sender;
    PhidgetLCD_clear(ch);
    [inputText0 setStringValue:@""];
    [inputText1 setStringValue:@""];
    
    if(button.state){
        NSMutableArray *smileyFace = [[NSMutableArray alloc] initWithObjects:
                                      @"          ",
                                      @"  ██  ██  ",
                                      @"          ",
                                      @"    ██    ",
                                      @"          ",
                                      @"██      ██",
                                      @"  ██████  ",
                                      @"          ",nil];
        NSMutableArray *sadFace = [[NSMutableArray alloc] initWithObjects:
                                   @"          ",
                                   @"  ██  ██  ",
                                   @"          ",
                                   @"    ██    ",
                                   @"          ",
                                   @"  ██████  ",
                                   @"██      ██",
                                   @"          ",nil];
        NSMutableArray *musicNotes = [[NSMutableArray alloc] initWithObjects:
                                      @"        ██",
                                      @"      ████",
                                      @"    ██  ██",
                                      @"  ██    ██",
                                      @"  ██    ██",
                                      @"  ██  ████",
                                      @"████  ████",
                                      @"████      ",nil];
        NSMutableArray *bell = [[NSMutableArray alloc] initWithObjects:
                                @"    ██    ",
                                @"  ██████  ",
                                @"  ██████  ",
                                @"  ██████  ",
                                @"██████████",
                                @"          ",
                                @"    ██    ",
                                @"          ",nil];
        NSMutableArray *heart = [[NSMutableArray alloc] initWithObjects:
                                 @"          ",
                                 @"  ██  ██  ",
                                 @"██████████",
                                 @"██████████",
                                 @"  ██████  ",
                                 @"    ██    ",
                                 @"          ",
                                 @"          ",nil];
        
        result = PhidgetLCD_setCharacterBitmap(ch, FONT_5x8, "\x8", [self getBitmapBytes:smileyFace]);
        if(result != EPHIDGET_OK){
            [self addError:result];
            return;
        }
        result = PhidgetLCD_setCharacterBitmap(ch, FONT_5x8, "\x9", [self getBitmapBytes:sadFace]);
        if(result != EPHIDGET_OK){
            [self addError:result];
            return;
        }
        result = PhidgetLCD_setCharacterBitmap(ch, FONT_5x8, "\xA", [self getBitmapBytes:musicNotes]);
        if(result != EPHIDGET_OK){
            [self addError:result];
            return;
        }
        result = PhidgetLCD_setCharacterBitmap(ch, FONT_5x8, "\xB", [self getBitmapBytes:bell]);
        if(result != EPHIDGET_OK){
            [self addError:result];
            return;
        }
        result = PhidgetLCD_setCharacterBitmap(ch, FONT_5x8, "\xC", [self getBitmapBytes:heart]);
        if(result != EPHIDGET_OK){
            [self addError:result];
            return;
        }
        result = PhidgetLCD_writeText(ch, FONT_5x8, 0, 0, "Custom \x8\x9\x8\x9 \xA \xB \xC");
        if(result != EPHIDGET_OK){
            [self addError:result];
            return;
        }
        result = PhidgetLCD_writeText(ch, FONT_5x8, 0, 1, "Characters! \x8\x9\x8\x9");
        if(result != EPHIDGET_OK){
            [self addError:result];
            return;
        }
        result = PhidgetLCD_flush(ch);
        if(result != EPHIDGET_OK){
            [self addError:result];
            return;
        }
    }
    else{
        result = PhidgetLCD_clear(ch);
        if(result != EPHIDGET_OK){
            [self addError:result];
            return;
        }
        result = PhidgetLCD_flush(ch);
        if(result != EPHIDGET_OK){
            [self addError:result];
            return;
        }
    }
    
}

@end
